@@ -1,6 +1,8 @@
+import email
 import boto3
 import botocore
 import os
+import jwt
 from numpy.core.fromnumeric import var
 import requests
 import uuid
@@ -14,21 +16,21 @@ import sys
 import base64
 import mimetypes
 import numpy as np
+import pandas as pd
 from aimodelshare.tools import extract_varnames_fromtrainingdata, _get_extension_from_filepath
 from aimodelshare.aws import get_s3_iam_client, run_function_on_lambda, get_token, get_aws_token, get_aws_client
 from aimodelshare.bucketpolicy import _custom_upload_policy
 from aimodelshare.exceptions import AuthorizationError, AWSAccessError, AWSUploadError
-from aimodelshare.api import create_prediction_api
 from aimodelshare.api import get_api_json
 from aimodelshare.modeluser import create_user_getkeyandpassword
 from aimodelshare.preprocessormodules import upload_preprocessor
 from aimodelshare.model import _get_predictionmodel_key, _extract_model_metadata
 from aimodelshare.data_sharing.share_data import share_data_codebuild
-from aimodelshare.containerization import clone_base_image
 from aimodelshare.aimsonnx import _get_metadata
 
-
-def take_user_info_and_generate_api(model_filepath, model_type, categorical,labels, preprocessor_filepath,custom_libraries, requirements, exampledata_json_filepath, repo_name, image_tag, reproducibility_env_filepath, memory, timeout):
+def take_user_info_and_generate_api(model_filepath, model_type, categorical,labels, preprocessor_filepath,
+                                    custom_libraries, requirements, exampledata_json_filepath, repo_name, 
+                                    image_tag, reproducibility_env_filepath, memory, timeout, pyspark_support=False):
     """
     Generates an api using model parameters and user credentials, from the user
 
@@ -94,10 +96,29 @@ def take_user_info_and_generate_api(model_filepath, model_type, categorical,labe
     api_id = response2['id']
     now = datetime.datetime.now()
     s3, iam, region = get_s3_iam_client(os.environ.get("AWS_ACCESS_KEY_ID"), os.environ.get("AWS_SECRET_ACCESS_KEY"), os.environ.get("AWS_REGION"))
-    s3["client"].create_bucket(
-        ACL='private',
-        Bucket=os.environ.get("BUCKET_NAME"))
+    
+    def create_bucket(s3_client, bucket_name, region):
+        try:
+            response=s3_client.head_bucket(Bucket=bucket_name)
+        except:
+            if(region=="us-east-1"):
+                response = s3_client.create_bucket(
+                    ACL="private",
+                    Bucket=bucket_name
+                )
+            else:
+                location={'LocationConstraint': region}
+                response=s3_client.create_bucket(
+                    ACL="private",
+                    Bucket=bucket_name,
+                    CreateBucketConfiguration=location
+                )
+        return response
+
+    create_bucket(s3['client'], os.environ.get("BUCKET_NAME"), region)
+
     # model upload
+
     Filepath = model_filepath
     model = onnx.load(model_filepath)
     metadata = _extract_model_metadata(model)
@@ -171,8 +192,11 @@ def take_user_info_and_generate_api(model_filepath, model_type, categorical,labe
     sys.stdout.flush()
     # }}}
     
+    from aimodelshare.api import create_prediction_api
     apiurl = create_prediction_api(model_filepath, unique_model_id,
-                                   model_type, categorical, labels,api_id,custom_libraries, requirements, repo_name, image_tag, memory, timeout)
+                                   model_type, categorical, labels,api_id,
+                                   custom_libraries, requirements, repo_name, 
+                                   image_tag, memory, timeout, pyspark_support=pyspark_support)
 
     finalresult = [apiurl["body"], apiurl["statusCode"],
                    now, unique_model_id, os.environ.get("BUCKET_NAME"), input_shape]
@@ -220,7 +244,8 @@ def upload_model_metadata(model, s3, bucket, model_id):
 
 def send_model_data_to_dyndb_and_return_api(api_info, private, categorical, preprocessor_filepath,
                                             aishare_modelname, aishare_modeldescription, aishare_modelevaluation, model_type,
-                                            aishare_tags, aishare_apicalls, exampledata_json_filepath,variablename_and_type_data="default"):
+                                            aishare_tags, aishare_apicalls, exampledata_json_filepath,
+                                            variablename_and_type_data="default", email_list=[]):
     """
     Updates dynamodb with model data taken as input from user along with already generated api info
     -----------
@@ -243,7 +268,10 @@ def send_model_data_to_dyndb_and_return_api(api_info, private, categorical, prep
                                 value- extracted from example_data
                                 [variable types,variable columns]
                                 'default' when training data info is not available to extract columns
-    
+    email_list: list of strings
+                values - list including all emails of users who have access the playground.
+                list should contain same emails used to sign up for modelshare.org account.
+                [OPTIONAL] to be set by the playground owner
     -----------
     Results
     print (api_info) : statements with the generated live prediction API information for the user
@@ -264,27 +292,29 @@ def send_model_data_to_dyndb_and_return_api(api_info, private, categorical, prep
         exampledata_addtodatabase={"exampledata":"TRUE"}
     else:
         exampledata_addtodatabase={"exampledata":"FALSE"}
-    bodydata = {"id": int(math.log(1/((time.time()*1000000)))*100000000000000),
-                "unique_model_id": unique_model_id,
-                "apideveloper": os.environ.get("username"),  # change this to first and last name
-                "apimodeldescription": aishare_modeldescription,
-                "apimodelevaluation": aishare_modelevaluation,
-                "apimodeltype": model_type,
-                # getting rid of extra quotes that screw up dynamodb string search on apiurls
-                "apiurl": api_info[0].strip('\"'),
-                "bucket_name": bucket_name,
-                "version": 1,
-                "modelname": aishare_modelname,
-                "tags": aishare_tags,
-                "Private": private,
-                "Categorical": categorical,
-                "delete": "FALSE",
-                "input_feature_dtypes": variablename_and_type_data[0],
-                "input_feature_names": variablename_and_type_data[1],
-                "preprocessor": preprocessor_filepath,
-                "preprocessor_fileextension": preprocessor_file_extension,
-                "input_shape": input_shape
-                }
+    bodydata = {
+        "id": int(math.log(1/((time.time()*1000000)))*100000000000000),
+        "unique_model_id": unique_model_id,
+        "apideveloper": os.environ.get("username"),  # change this to first and last name
+        "apimodeldescription": aishare_modeldescription,
+        "apimodelevaluation": aishare_modelevaluation,
+        "apimodeltype": model_type,
+        # getting rid of extra quotes that screw up dynamodb string search on apiurls
+        "apiurl": api_info[0].strip('\"'),
+        "bucket_name": bucket_name,
+        "version": 1,
+        "modelname": aishare_modelname,
+        "tags": aishare_tags,
+        "Private": private,
+        "Categorical": categorical,
+        "delete": "FALSE",
+        "input_feature_dtypes": variablename_and_type_data[0],
+        "input_feature_names": variablename_and_type_data[1],
+        "preprocessor": preprocessor_filepath,
+        "preprocessor_fileextension": preprocessor_file_extension,
+        "input_shape": input_shape,
+        "email_list": email_list,
+    }
     bodydata.update(exampledata_addtodatabase)
     # Get the response
     headers_with_authentication = {'Content-Type': 'application/json', 'authorizationToken': os.environ.get("JWT_AUTHORIZATION_TOKEN"), 'Access-Control-Allow-Headers':
@@ -319,7 +349,10 @@ def send_model_data_to_dyndb_and_return_api(api_info, private, categorical, prep
     return print("\n\n" + finalresult2 + "\n" + final_message + web_dashboard_url)
 
 
-def model_to_api(model_filepath, model_type, private, categorical, y_train, preprocessor_filepath, custom_libraries="FALSE", example_data=None, image="", base_image_api_endpoint="https://vupwujn586.execute-api.us-east-1.amazonaws.com/dev/copybasetouseracct", update=False, reproducibility_env_filepath=None, memory=None, timeout=None):
+def model_to_api(model_filepath, model_type, private, categorical, y_train, preprocessor_filepath, 
+                custom_libraries="FALSE", example_data=None, image="", 
+                base_image_api_endpoint="https://vupwujn586.execute-api.us-east-1.amazonaws.com/dev/copybasetouseracct", 
+                update=False, reproducibility_env_filepath=None, memory=None, timeout=None, , email_list=[],pyspark_support=False):
     """
       Launches a live prediction REST API for deploying ML models using model parameters and user credentials, provided by the user
       Inputs : 8
@@ -367,6 +400,10 @@ def model_to_api(model_filepath, model_type, private, categorical, y_train, prep
                                 [OPTIONAL] to be set by the user
                                 "./reproducibility.json" 
                                 file is generated using export_reproducibility_env function from the AI Modelshare library
+      email_list: list of strings
+                values - list including all emails of users who have access the playground.
+                list should contain same emails used to sign up for modelshare.org account.
+                [OPTIONAL] to be set by the playground owner
       -----------
       Returns
       print_api_info : prints statements with generated live prediction API details
@@ -383,6 +420,15 @@ def model_to_api(model_filepath, model_type, private, categorical, y_train, prep
                                          aws_secret_access_key = os.environ.get("AWS_SECRET_ACCESS_KEY"), 
                                          region_name=os.environ.get("AWS_REGION"))
 
+    if all([isinstance(email_list, list)]):
+        idtoken = get_aws_token()
+        decoded = jwt.decode(idtoken, options={"verify_signature": False})  # works in PyJWT < v2.0
+        email = decoded['email']
+        # Owner has to be the first on the list
+        email_list.insert(0, email)
+    else:
+        return print("email_list argument empty or incorrectly formatted. Please provide a list of emails for authorized competition participants formatted as strings.")
+
     if(image!=""):
         repo_name, image_tag = image.split(':')
     elif model_type=="tabular":
@@ -395,7 +441,12 @@ def model_to_api(model_filepath, model_type, private, categorical, y_train, prep
         repo_name, image_tag = "aimodelshare_base_image", "v3"
     else:
         repo_name, image_tag = "aimodelshare_base_image", "v3"
+
+    # Pyspark mode
+    if pyspark_support:
+        repo_name, image_tag = "aimodelshare_base_image", "pyspark"
     
+    from aimodelshare.containerization import clone_base_image
     response = clone_base_image(user_session, repo_name, image_tag, "517169013426", base_image_api_endpoint, update)
     if(response["Status"]==0):
         print(response["Success"])
@@ -421,7 +472,7 @@ def model_to_api(model_filepath, model_type, private, categorical, y_train, prep
 
     # Force user to provide example data for tabular models {{{
     if any([model_type.lower() == "tabular", model_type.lower() == "timeseries"]):
-        if example_data is None:
+        if not isinstance(example_data, pd.DataFrame):
             return print("Error: Example data is required for tabular models. \n Please provide a pandas DataFrame with a sample of your X data (in the format expected by your preprocessor) and try again.")
     else:
         pass
@@ -453,9 +504,12 @@ def model_to_api(model_filepath, model_type, private, categorical, y_train, prep
     sys.stdout.write("[===                                  ] Progress: 5% - Accessing Amazon Web Services, uploading resources...")
     sys.stdout.flush()
     # }}}
-    
+
     api_info = take_user_info_and_generate_api( 
-        model_filepath, model_type, categorical, labels,preprocessor_filepath,custom_libraries, requirements, exampledata_json_filepath, repo_name, image_tag, reproducibility_env_filepath, memory, timeout)
+        model_filepath, model_type, categorical, labels, 
+        preprocessor_filepath, custom_libraries, requirements, 
+        exampledata_json_filepath, repo_name, image_tag, 
+        reproducibility_env_filepath, memory, timeout, pyspark_support=pyspark_support)
 
     ### Progress Update #5/6 {{{
     sys.stdout.write('\r')
@@ -466,11 +520,12 @@ def model_to_api(model_filepath, model_type, private, categorical, y_train, prep
     print_api_info = send_model_data_to_dyndb_and_return_api(
         api_info, private, categorical,preprocessor_filepath, aishare_modelname,
         aishare_modeldescription, aishare_modelevaluation, model_type,
-        aishare_tags, aishare_apicalls, exampledata_json_filepath,variablename_and_type_data)
+        aishare_tags, aishare_apicalls, exampledata_json_filepath,
+        variablename_and_type_data, email_list)
     
     return api_info[0]
 
-def create_competition(apiurl, data_directory, y_test, eval_metric_filepath=None, email_list=[], public=False):
+def create_competition(apiurl, data_directory, y_test, eval_metric_filepath=None, email_list=[], public=False, public_private_split=0.5):
     """
     Creates a model competition for a deployed prediction REST API
     Inputs : 4
@@ -536,7 +591,7 @@ def create_competition(apiurl, data_directory, y_test, eval_metric_filepath=None
 
 
     pickle.dump(y_test,open(ytest_path,"wb"))
-    s3["client"].upload_file(ytest_path, os.environ.get("BUCKET_NAME"),  model_id + "/ytest.pkl")
+    s3["client"].upload_file(ytest_path, os.environ.get("BUCKET_NAME"),  model_id + "/competition/ytest.pkl")
 
 
     if eval_metric_filepath is not None:
@@ -547,7 +602,7 @@ def create_competition(apiurl, data_directory, y_test, eval_metric_filepath=None
 
                 eval_metric_name = i.split('/')[-1]
 
-                s3["client"].upload_file(i, os.environ.get("BUCKET_NAME"),  model_id + '/metrics_' + eval_metric_name)
+                s3["client"].upload_file(i, os.environ.get("BUCKET_NAME"),  model_id + '/competition/metrics_' + eval_metric_name)
 
         else:
 
@@ -555,7 +610,7 @@ def create_competition(apiurl, data_directory, y_test, eval_metric_filepath=None
 
             print(eval_metric_name)
 
-            s3["client"].upload_file(eval_metric_filepath, os.environ.get("BUCKET_NAME"), model_id + '/metrics_' + eval_metric_name)
+            s3["client"].upload_file(eval_metric_filepath, os.environ.get("BUCKET_NAME"), model_id + '/competition/metrics_' + eval_metric_name)
 
 
     # get api_id from apiurl, generate txt file name
@@ -584,7 +639,8 @@ def create_competition(apiurl, data_directory, y_test, eval_metric_filepath=None
     datauri=share_data_codebuild(account_number,os.environ.get("AWS_REGION"),data_directory)
     
     #create and upload json file with list of authorized users who can submit to this competition.
-    _create_competitionuserauth_json(apiurl, email_list,public,datauri['ecr_uri'])
+    _create_competitionuserauth_json(apiurl, email_list,public,datauri['ecr_uri'], submission_type="competition")
+    _create_public_private_split_json(apiurl, public_private_split, "competition")
 
     bodydata = {"unique_model_id": model_id,
                 "bucket_name": api_bucket,
@@ -619,7 +675,205 @@ def create_competition(apiurl, data_directory, y_test, eval_metric_filepath=None
   
     return print(final_message)
 
-def _create_competitionuserauth_json(apiurl, email_list=[],public=False, datauri=None): 
+
+
+
+def create_experiment(apiurl, data_directory, y_test, eval_metric_filepath=None, email_list=[], public=False, public_private_split=0.5):
+    """
+    Creates a model experiment for a deployed prediction REST API
+    Inputs : 4
+    Output : Create ML model experiment and allow authorized users to submit models to resulting leaderboard/competition
+    
+    ---------
+    Parameters
+    
+    apiurl: string
+            URL of deployed prediction API 
+    
+    y_test :  list of y values for test data used to generate metrics from predicted values from X test data submitted via the submit_model() function
+                [REQUIRED] to generate eval metrics in competition leaderboard
+            
+    data_directory : folder storing training data and test data (excluding Y test data)
+    email_list: [REQUIRED] list of comma separated emails for users who are allowed to submit models to experiment.  Emails should be strings in a list.
+                                          
+    ---------
+    Returns
+    finalmessage : Information such as how to submit models to competition
+
+    """
+    if all([isinstance(email_list, list)]):
+        if any([len(email_list)>0, public=="True",public=="TRUE",public==True]):
+              import jwt
+              idtoken=get_aws_token()
+              decoded = jwt.decode(idtoken, options={"verify_signature": False})  # works in PyJWT < v2.0
+              email=decoded['email']
+              email_list.append(email)
+    else:
+        return print("email_list argument empty or incorrectly formatted.  Please provide a list of emails for authorized competition participants formatted as strings.")
+
+    # create temporary folder
+    temp_dir = tempfile.gettempdir()
+    
+    s3, iam, region = get_s3_iam_client(os.environ.get("AWS_ACCESS_KEY_ID"), os.environ.get("AWS_SECRET_ACCESS_KEY"), os.environ.get("AWS_REGION"))
+    
+    # Get bucket and model_id subfolder for user based on apiurl {{{
+    response, error = run_function_on_lambda(
+        apiurl, **{"delete": "FALSE", "versionupdateget": "TRUE"}
+    )
+    if error is not None:
+        raise error
+
+    _, api_bucket, model_id = json.loads(response.content.decode("utf-8"))
+    # }}} 
+    
+    # upload y_test data: 
+    ytest_path = os.path.join(temp_dir, "ytest.pkl")
+    import pickle
+    #ytest data to load to s3
+
+    if y_test is not None:
+        if type(y_test) is not list:
+            y_test=y_test.tolist()
+        else: 
+            pass
+
+        if all(isinstance(x, (np.float64)) for x in y_test):
+              y_test = [float(i) for i in y_test]
+        else: 
+            pass
+
+
+    pickle.dump(y_test,open(ytest_path,"wb"))
+    s3["client"].upload_file(ytest_path, os.environ.get("BUCKET_NAME"),  model_id + "/experiment/ytest.pkl")
+
+
+    if eval_metric_filepath is not None:
+    
+        if isinstance(eval_metric_filepath, list):
+
+            for i in eval_metric_filepath: 
+
+                eval_metric_name = i.split('/')[-1]
+
+                s3["client"].upload_file(i, os.environ.get("BUCKET_NAME"),  model_id + '/experiment/metrics_' + eval_metric_name)
+
+        else:
+
+            eval_metric_name = eval_metric_filepath.split('/')[-1]
+
+            print(eval_metric_name)
+
+            s3["client"].upload_file(eval_metric_filepath, os.environ.get("BUCKET_NAME"), model_id + '/experiment/metrics_' + eval_metric_name)
+
+
+    # get api_id from apiurl, generate txt file name
+    api_url_trim = apiurl.split('https://')[1]
+    api_id = api_url_trim.split(".")[0]
+
+    print("\n--INPUT COMPETITION DETAILS--\n")
+
+    aishare_competitionname = input("Enter experiment name:")
+    aishare_competitiondescription = input("Enter experiment description:")
+
+    print("\n--INPUT DATA DETAILS--\n")
+    print("Note: (optional) Save an optional LICENSE.txt file in your experiment data directory to make users aware of any restrictions on data sharing/usage.\n")
+
+    aishare_datadescription = input(
+        "Enter data description (i.e.- filenames denoting training and test data, file types, and any subfolders where files are stored):")
+    
+    aishare_datalicense = input(
+        "Enter optional data license descriptive name (e.g.- 'MIT, Apache 2.0, CC0, Other, etc.'):")    
+    user_session = boto3.session.Session(aws_access_key_id=os.environ.get("AWS_ACCESS_KEY_ID"),
+                                          aws_secret_access_key = os.environ.get("AWS_SECRET_ACCESS_KEY"), 
+                                         region_name=os.environ.get("AWS_REGION"))
+    account_number = user_session.client(
+        'sts').get_caller_identity().get('Account')
+
+    datauri=share_data_codebuild(account_number,os.environ.get("AWS_REGION"),data_directory)
+    
+    #create and upload json file with list of authorized users who can submit to this competition.
+    _create_competitionuserauth_json(apiurl, email_list,public,datauri['ecr_uri'], submission_type="experiment")
+    _create_public_private_split_json(apiurl, public_private_split, "experiment")
+
+    bodydata = {"unique_model_id": model_id,
+                "bucket_name": api_bucket,
+                "apideveloper": os.environ.get("username"),  # change this to first and last name
+                "experiment":"TRUE",
+                "competitionname":aishare_competitionname,                
+                "competitiondescription": aishare_competitiondescription,
+                # getting rid of extra quotes that screw up dynamodb string search on apiurls
+                "apiurl": apiurl,
+                "version": 0,
+                "Private": "FALSE",
+                "delete": "FALSE",
+                'datadescription':aishare_datadescription,
+                'dataecruri':datauri['ecr_uri'],
+                 'datalicense': aishare_datalicense}
+    
+    # Get the response
+    headers_with_authentication = {'Content-Type': 'application/json', 'authorizationToken': os.environ.get("JWT_AUTHORIZATION_TOKEN"), 'Access-Control-Allow-Headers':
+                                   'Content-Type,X-Amz-Date,authorizationToken,Access-Control-Allow-Origin,X-Api-Key,X-Amz-Security-Token,Authorization', 'Access-Control-Allow-Origin': '*'}
+    # modeltoapi lambda function invoked through below url to return new prediction api in response
+    requests.post("https://o35jwfakca.execute-api.us-east-1.amazonaws.com/dev/modeldata",
+                  json=bodydata, headers=headers_with_authentication)
+
+      
+    final_message = ("\n Success! Model experiment created. \n\n"
+                "You may now update your prediction API runtime model and verify evaluation metrics with the update_runtime_model() function.\n\n"
+                "To upload new models and/or preprocessors to this API, team members should use \n"
+                "the following credentials:\n\napiurl='" + apiurl+"'"+"\nfrom aimodelshare.aws import set_credentials\nset_credentials(apiurl=apiurl)\n\n"
+                "They can then submit models to your experiment by using the following code: \n\nexperiment= ai.Experiment(apiurl)\n"
+                "download_data('"+datauri['ecr_uri']+"') \n"
+                 "# Use this data to preprocess data and train model. Write and save preprocessor fxn, save model to onnx file, generate predicted y values\n using X test data, then submit a model below.\n\n"
+                "experiment.submit_model(model_filepath, preprocessor_filepath, prediction_submission_list)")
+  
+    return print(final_message)
+
+def _create_public_private_split_json(apiurl, split=0.5, submission_type='competition'): 
+      import json
+      if all(["AWS_ACCESS_KEY_ID" in os.environ, 
+            "AWS_SECRET_ACCESS_KEY" in os.environ,
+            "AWS_REGION" in os.environ,
+           "username" in os.environ, 
+           "password" in os.environ]):
+        pass
+      else:
+          return print("'Set public-private split' unsuccessful. Please provide credentials with set_credentials().")
+
+      # Create user session
+      aws_client=get_aws_client(aws_key=os.environ.get('AWS_ACCESS_KEY_ID'), 
+                                aws_secret=os.environ.get('AWS_SECRET_ACCESS_KEY'), 
+                                aws_region=os.environ.get('AWS_REGION'))
+      
+      user_sess = boto3.session.Session(aws_access_key_id=os.environ.get('AWS_ACCESS_KEY_ID'), 
+                                        aws_secret_access_key=os.environ.get('AWS_SECRET_ACCESS_KEY'), 
+                                        region_name=os.environ.get('AWS_REGION'))
+      
+      s3 = user_sess.resource('s3')
+
+      # Get bucket and model_id for user based on apiurl {{{
+      response, error = run_function_on_lambda(
+          apiurl, **{"delete": "FALSE", "versionupdateget": "TRUE"}
+      )
+      if error is not None:
+          raise error
+
+      _, api_bucket, model_id = json.loads(response.content.decode("utf-8"))
+      # }}}
+      
+      import json  
+      import tempfile
+      tempdir = tempfile.TemporaryDirectory()
+      with open(tempdir.name+'/public_private_split.json', 'w', encoding='utf-8') as f:
+          json.dump({"public_private_split": str(split)}, f, ensure_ascii=False, indent=4)
+
+      aws_client['client'].upload_file(
+            tempdir.name+"/public_private_split.json", api_bucket, model_id +"/"+submission_type+"/public_private_split.json"
+        )
+      
+      return
+
+def _create_competitionuserauth_json(apiurl, email_list=[],public=False, datauri=None, submission_type="competition"): 
       import json
       if all(["AWS_ACCESS_KEY_ID" in os.environ, 
             "AWS_SECRET_ACCESS_KEY" in os.environ,
@@ -659,12 +913,69 @@ def _create_competitionuserauth_json(apiurl, email_list=[],public=False, datauri
           json.dump({"emaillist": email_list, "public":str(public).upper(),"datauri":str(datauri)}, f, ensure_ascii=False, indent=4)
 
       aws_client['client'].upload_file(
-            tempdir.name+"/competitionuserdata.json", api_bucket, model_id + "/competitionuserdata.json"
+            tempdir.name+"/competitionuserdata.json", api_bucket, model_id +"/"+submission_type+"/competitionuserdata.json"
         )
       
       return
 
-def update_access_list(apiurl, email_list=[],update_type="Add"): 
+def update_playground_access_list(apiurl, email_list=[], update_type="Add"): 
+    """
+    Updates list of authenticated participants who can submit new models to a competition.
+    ---------------
+    Parameters:
+    apiurl: string
+            URL of deployed prediction API 
+    
+    email_list: [REQUIRED] list of comma separated emails for users who are allowed to access model playground.  Emails should be strings in a list.
+    update_type:[REQUIRED] options, string: 'Add', 'Remove', 'Replace'. Add appends user emails to original list, Remove deletes users from list, 
+                and 'Replace' overwrites the original list with the new list provided.    
+    -----------------
+    Returns
+    response:   "Success" upon successful request
+    """
+    if update_type not in ['Add', 'Remove', 'Replace']:
+        return "Error: update_type must be in the form of 'Add', 'Remove', or 'Replace'"
+    
+    bodydata = {
+        "modifyaccess": "TRUE",
+        "apideveloper": os.environ.get("username"),  # change this to first and last name
+        "apiurl": apiurl,
+        "operation": update_type,
+        "email_list": email_list,
+    }
+
+    # Get the response
+    headers_with_authentication = {'Content-Type': 'application/json', 'authorizationToken': os.environ.get("JWT_AUTHORIZATION_TOKEN"), 'Access-Control-Allow-Headers':
+                                'Content-Type,X-Amz-Date,authorizationToken,Access-Control-Allow-Origin,X-Api-Key,X-Amz-Security-Token,Authorization', 'Access-Control-Allow-Origin': '*'}
+    # modeltoapi lambda function invoked through below url to return new prediction api in response
+    response = requests.post("https://bhrdesksak.execute-api.us-east-1.amazonaws.com/dev/modeldata",
+                            json=bodydata, headers=headers_with_authentication)
+    response_string = response.text
+    response_string = response_string[1:-1]
+
+    # Build output {{{
+    finalresult = "Success! Your Model Playground email list has been updated. Playground Url: " + apiurl
+    return finalresult
+
+def get_playground_access_list(apiurl):
+    bodydata = {
+        "getaccess": "TRUE",
+        "apideveloper": os.environ.get("username"),  # change this to first and last name
+        "apiurl": apiurl,
+    }
+
+    # Get the response
+    headers_with_authentication = {'Content-Type': 'application/json', 'authorizationToken': os.environ.get("JWT_AUTHORIZATION_TOKEN"), 'Access-Control-Allow-Headers':
+                                'Content-Type,X-Amz-Date,authorizationToken,Access-Control-Allow-Origin,X-Api-Key,X-Amz-Security-Token,Authorization', 'Access-Control-Allow-Origin': '*'}
+    # modeltoapi lambda function invoked through below url to return new prediction api in response
+    response = requests.post("https://bhrdesksak.execute-api.us-east-1.amazonaws.com/dev/modeldata",
+                            json=bodydata, headers=headers_with_authentication)
+    response_string = response.text
+    response_string = response_string[1:-1]
+
+    return response_string.split(',')
+
+def update_access_list(apiurl, email_list=[],update_type="Add", submission_type="competition"):
       """
       Updates list of authenticated participants who can submit new models to a competition.
       ---------------
@@ -724,7 +1035,7 @@ def update_access_list(apiurl, email_list=[],update_type="Add"):
           import json  
           import tempfile
           tempdir = tempfile.TemporaryDirectory()
-          content_object = aws_client['resource'].Object(bucket_name=api_bucket, key=model_id + "/competitionuserdata.json")
+          content_object = aws_client['resource'].Object(bucket_name=api_bucket, key=model_id +"/"+submission_type+"/competitionuserdata.json")
           file_content = content_object.get()['Body'].read().decode('utf-8')
           json_content = json.loads(file_content)
           json_content['emaillist']=email_list
@@ -732,13 +1043,13 @@ def update_access_list(apiurl, email_list=[],update_type="Add"):
               json.dump(json_content, f, ensure_ascii=False, indent=4)
 
           aws_client['client'].upload_file(
-                tempdir.name+"/competitionuserdata.json", api_bucket, model_id + "/competitionuserdata.json"
+                tempdir.name+"/competitionuserdata.json", api_bucket, model_id +"/"+submission_type+"/competitionuserdata.json"
             )
       elif update_type=="Add":
           import json  
           import tempfile
           tempdir = tempfile.TemporaryDirectory()
-          content_object = aws_client['resource'].Object(bucket_name=api_bucket, key=model_id + "/competitionuserdata.json")
+          content_object = aws_client['resource'].Object(bucket_name=api_bucket, key=model_id +"/"+submission_type+"/competitionuserdata.json")
           file_content = content_object.get()['Body'].read().decode('utf-8')
           json_content = json.loads(file_content)
 
@@ -751,7 +1062,7 @@ def update_access_list(apiurl, email_list=[],update_type="Add"):
               json.dump(json_content, f, ensure_ascii=False, indent=4)
 
           aws_client['client'].upload_file(
-                tempdir.name+"/competitionuserdata.json", api_bucket, model_id + "/competitionuserdata.json"
+                tempdir.name+"/competitionuserdata.json", api_bucket, model_id +"/"+submission_type+"/competitionuserdata.json"
             )
      
           return "Success: Your competition participant access list is now updated."
@@ -761,7 +1072,7 @@ def update_access_list(apiurl, email_list=[],update_type="Add"):
           tempdir = tempfile.TemporaryDirectory()
     
           aws_client['resource']
-          content_object = aws_client['resource'].Object(bucket_name=api_bucket, key=model_id + "/competitionuserdata.json")
+          content_object = aws_client['resource'].Object(bucket_name=api_bucket, key=model_id +"/"+submission_type+"/competitionuserdata.json")
           file_content = content_object.get()['Body'].read().decode('utf-8')
           json_content = json.loads(file_content)
 
@@ -774,7 +1085,7 @@ def update_access_list(apiurl, email_list=[],update_type="Add"):
               json.dump(json_content, f, ensure_ascii=False, indent=4)
 
           aws_client['client'].upload_file(
-                tempdir.name+"/competitionuserdata.json", api_bucket, model_id + "/competitionuserdata.json"
+                tempdir.name+"/competitionuserdata.json", api_bucket, model_id +"/"+submission_type+"/competitionuserdata.json"
             )
           return "Success: Your competition participant access list is now updated."
       elif update_type=="Get":
@@ -783,7 +1094,7 @@ def update_access_list(apiurl, email_list=[],update_type="Add"):
           tempdir = tempfile.TemporaryDirectory()
 
           aws_client['resource']
-          content_object = aws_client['resource'].Object(bucket_name=api_bucket, key=model_id + "/competitionuserdata.json")
+          content_object = aws_client['resource'].Object(bucket_name=api_bucket, key=model_id +"/"+submission_type+"/competitionuserdata.json")
           file_content = content_object.get()['Body'].read().decode('utf-8')
           json_content = json.loads(file_content)
           return json_content['emaillist']
